      SUBROUTINE JIK_BLOCK( M, N, A, LDA, IPIV, INFO, NB )
*
*  -- LAPACK routine (preliminary version) --
*     Univ. of Tennessee, Oak Ridge National Lab, Argonne National Lab,
*     Courant Institute, NAG Ltd., and Rice University
*     March 26, 1990
*
*     .. Scalar Arguments ..
      INTEGER            INFO, LDA, M, N
*     ..
*     .. Array Arguments ..
      INTEGER            IPIV( * )
      DOUBLE PRECISION   A( LDA, * )
*     ..
*
*  Purpose
*  =======
*
*  DGETRF computes an LU factorization of a general m-by-n matrix A
*  using partial pivoting with row interchanges.
*
*  The factorization has the form
*     A = P * L * U
*  where P is a permutation matrix, L is lower triangular with unit
*  diagonal elements (lower trapezoidal if m > n), and U is upper
*  triangular (upper trapezoidal if m < n).
*
*  This is the Level 3 BLAS version of the Crout algorithm.
*
*  Arguments
*  =========
*
*  M       (input) INTEGER
*          The number of rows of the matrix A.  M >= 0.
*
*  N       (input) INTEGER
*          The number of columns of the matrix A.  N >= 0.
*
*  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N)
*          On entry, the m x n matrix to be factored.
*          On exit, the factors L and U; the unit diagonal elements of L
*          are not stored.
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.  LDA >= max(1,M).
*
*  IPIV    (output) INTEGER array, dimension (min(M,N))
*          The pivot indices.  Row i of the matrix was interchanged with
*          row IPIV(i).
*
*  INFO    (output) INTEGER
*          = 0: successful exit
*          < 0: if INFO = -k, the k-th argument had an illegal value
*          > 0: if INFO = k, U(k,k) is exactly zero. The factorization
*               has been completed, but the factor U is exactly
*               singular, and division by zero will occur if it is used
*               to solve a system of equations or to compute the inverse
*               of A.
*
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION   ONE
      PARAMETER          ( ONE = 1.0D+0 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, J, JB, NB
*     ..
*     .. External Subroutines ..
      EXTERNAL           DGEMM, DLASWP, DTRSM, XERBLA
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, MIN
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
      IF( M.LT.0 ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( LDA.LT.MAX( 1, M ) ) THEN
         INFO = -4
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'JIK_BLOCK', -INFO )
         RETURN
      END IF
*
*     Quick return if possible.
*
      IF( M.EQ.0 .OR. N.EQ.0 )
     $   RETURN
*
*
      IF( NB.EQ.1 ) THEN
c         CALL DGETF2( M, N, A, LDA, IPIV, INFO )
         CALL JIK_NOBLOCK( M, N, A, LDA, IPIV, INFO )
      ELSE
*
         DO 20 J = 1, MIN( M, N ), NB
            JB = MIN( MIN( M, N )-J+1, NB )
*
*           Update diagonal and subdiagonal blocks.
*
            CALL DGEMM( 'No transpose', 'No transpose', M-J+1, JB, J-1,
     $                  -ONE, A( J, 1 ), LDA, A( 1, J ), LDA, ONE,
     $                  A( J, J ), LDA )
*
*           Factorize diagonal and subdiagonal blocks and test for
*           singularity.
*
c            CALL DGETF2( M-J+1, JB, A( J, J ), LDA, IPIV( J ), INFO )
            CALL JIK_NOBLOCK(M-J+1,JB,A( J, J ),LDA,IPIV( J ),INFO)
*
*           Update pivot indices and apply the interchanges to the
*           columns on either side of the current block.
*
            DO 10 I = J, MIN( M, J+JB-1 )
               IPIV( I ) = J - 1 + IPIV( I )
   10       CONTINUE
            CALL DLASWP( J-1, A, LDA, J, J+JB-1, IPIV, 1 )
            CALL DLASWP( N-J-JB+1, A( 1, J+JB ), LDA, J, J+JB-1, IPIV,
     $                   1 )
*
            IF( INFO.NE.0 )
     $         GO TO 30
*
            IF( J+JB.LE.N ) THEN
*
*              Compute block row of U.
*
               CALL DGEMM( 'No transpose', 'No transpose', JB, N-J-JB+1,
     $                     J-1, -ONE, A( J, 1 ), LDA, A( 1, J+JB ), LDA,
     $                     ONE, A( J, J+JB ), LDA )
               CALL DTRSM( 'Left', 'Lower', 'No transpose', 'Unit', JB,
     $                     N-J-JB+1, ONE, A( J, J ), LDA, A( J, J+JB ),
     $                     LDA )
            END IF
   20    CONTINUE
      END IF
      RETURN
*
   30 CONTINUE
      INFO = INFO + J - 1
      RETURN
*
*     End of DGETRF
*
      END
